@@ -14,7 +14,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* ---------------------------- included header files ----------------------- */
+/* ---------------------------- included header files ---------------------- */
 
 #include "config.h"
 
@@ -23,6 +23,8 @@
 
 #include "libs/fvwmlib.h"
 #include "libs/FShape.h"
+#include "libs/charmap.h"
+#include "libs/wcontext.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "execcontext.h"
@@ -34,16 +36,18 @@
 #include "focus.h"
 #include "borders.h"
 #include "frame.h"
+#include "gnome.h"
+#include "ewmh.h"
 
-/* ---------------------------- local definitions --------------------------- */
+/* ---------------------------- local definitions -------------------------- */
 
-/* ---------------------------- local macros -------------------------------- */
+/* ---------------------------- local macros ------------------------------- */
 
-/* ---------------------------- imports ------------------------------------- */
+/* ---------------------------- imports ------------------------------------ */
 
-/* ---------------------------- included code files ------------------------- */
+/* ---------------------------- included code files ------------------------ */
 
-/* ---------------------------- local types --------------------------------- */
+/* ---------------------------- local types -------------------------------- */
 
 typedef struct
 {
@@ -72,7 +76,7 @@ typedef struct
 	Window w_with_focus;
 	int current_step;
 	int curr_titlebar_compression;
-	direction_type shade_dir;
+	direction_t shade_dir;
 	window_parts trans_parts;
 	struct
 	{
@@ -105,9 +109,9 @@ typedef struct
 	} step_flags;
 } mr_args_internal;
 
-/* ---------------------------- forward declarations ------------------------ */
+/* ---------------------------- forward declarations ----------------------- */
 
-/* ---------------------------- local variables ----------------------------- */
+/* ---------------------------- local variables ---------------------------- */
 
 /* windows used to hide animation steps */
 static struct
@@ -116,9 +120,9 @@ static struct
 	Window w[4];
 } hide_wins;
 
-/* ---------------------------- exported variables (globals) ---------------- */
+/* ---------------------------- exported variables (globals) --------------- */
 
-/* ---------------------------- local functions ----------------------------- */
+/* ---------------------------- local functions ---------------------------- */
 
 #if 0
 static void print_g(char *text, rectangle *g)
@@ -165,9 +169,9 @@ static void combine_decor_gravities(
 }
 
 static void get_resize_decor_gravities_one_axis(
-	frame_decor_gravities_type *ret_grav, direction_type title_dir,
-	frame_move_resize_mode axis_mode, direction_type neg_dir,
-	direction_type pos_dir, int is_moving)
+	frame_decor_gravities_type *ret_grav, direction_t title_dir,
+	frame_move_resize_mode axis_mode, direction_t neg_dir,
+	direction_t pos_dir, int is_moving)
 {
 
 	int title_grav;
@@ -312,7 +316,7 @@ static void frame_setup_titlebar(
 	FvwmWindow *fw, rectangle *frame_g, window_parts setup_parts,
 	rectangle *diff_g)
 {
-	frame_title_layout_type title_layout;
+	frame_title_layout_t title_layout;
 	int i;
 
 	if (!HAS_TITLE(fw))
@@ -518,13 +522,13 @@ static int frame_get_titlebar_compression(
  *   The mode for the resize operation
  */
 static void frame_get_resize_decor_gravities(
-	frame_decor_gravities_type *ret_grav, direction_type title_dir,
+	frame_decor_gravities_type *ret_grav, direction_t title_dir,
 	frame_move_resize_mode rmode, rectangle *delta_g)
 {
 	frame_decor_gravities_type grav_x;
 	frame_decor_gravities_type grav_y;
-	direction_type title_dir_x;
-	direction_type title_dir_y;
+	direction_t title_dir_x;
+	direction_t title_dir_y;
 
 	gravity_split_xy_dir(&title_dir_x, &title_dir_y, title_dir);
 	get_resize_decor_gravities_one_axis(
@@ -658,8 +662,8 @@ static rectangle *frame_get_hidden_pos(
 	rectangle *ret_hidden_g)
 {
 	rectangle *target_g;
-	direction_type dir_x;
-	direction_type dir_y;
+	direction_t dir_x;
+	direction_t dir_y;
 
 	if (do_unhide == False)
 	{
@@ -1178,7 +1182,69 @@ static int frame_get_shading_laziness(
 	}
 }
 
-/* ---------------------------- interface functions ------------------------- */
+static void frame_reshape_border(FvwmWindow *fw)
+{
+	int dw;
+	int dh;
+	int grav;
+	int off_x = 0;
+	int off_y = 0;
+	rectangle naked_g;
+	rectangle *new_g;
+	size_borders b_old;
+	size_borders b_new;
+
+	get_window_borders(fw, &b_old);
+	dw = b_new.total_size.width - b_old.total_size.width;
+	dh = b_new.total_size.height - b_old.total_size.height;
+	/* calculate the new offsets */
+	if (!IS_MAXIMIZED(fw))
+	{
+		grav = fw->hints.win_gravity;
+		new_g = &fw->normal_g;
+	}
+	else
+	{
+		/* maximized windows are always considered to have
+		 * NorthWestGravity */
+		grav = NorthWestGravity;
+		new_g = &fw->max_g;
+		off_x = fw->normal_g.x - fw->max_g.x;
+		off_y = fw->normal_g.y - fw->max_g.y;
+	}
+	gravity_get_naked_geometry(grav, fw, &naked_g, new_g);
+	gravity_translate_to_northwest_geometry_no_bw(
+		grav, fw, &naked_g, &naked_g);
+	set_window_border_size(fw, fw->unshaped_boundary_width);
+	get_window_borders(fw, &b_new);
+	gravity_add_decoration(grav, fw, new_g, &naked_g);
+	if (IS_MAXIMIZED(fw))
+	{
+		/* prevent random paging when unmaximizing after the border
+		 * width has changed */
+		fw->max_offset.x += fw->normal_g.x - fw->max_g.x - off_x;
+		fw->max_offset.y += fw->normal_g.y - fw->max_g.y - off_y;
+	}
+	if (IS_SHADED(fw))
+	{
+		get_unshaded_geometry(fw, new_g);
+		if (USED_TITLE_DIR_FOR_SHADING(fw))
+		{
+			SET_SHADED_DIR(fw, GET_TITLE_DIR(fw));
+		}
+		get_shaded_geometry(fw, &fw->frame_g, new_g);
+	}
+	else
+	{
+		get_relative_geometry(&fw->frame_g, new_g);
+	}
+	frame_force_setup_window(
+		fw, new_g->x, new_g->y, new_g->width, new_g->height, True);
+
+	return;
+}
+
+/* ---------------------------- interface functions ------------------------ */
 
 /* Initialise structures local to frame.c */
 void frame_init(void)
@@ -1247,7 +1313,7 @@ void frame_destroyed_frame(
 
 void frame_get_titlebar_dimensions(
 	FvwmWindow *fw, rectangle *frame_g, rectangle *diff_g,
-	frame_title_layout_type *title_layout)
+	frame_title_layout_t *title_layout)
 {
 	size_borders b;
 	int i;
@@ -1679,7 +1745,7 @@ frame_move_resize_args frame_create_move_resize_args(
 		mra->flags.do_not_configure_client = 1;
 	}
 	mra->mode = mr_mode;
-	mra->shade_dir = (direction_type)shade_dir;
+	mra->shade_dir = (direction_t)shade_dir;
 	mra->w_with_focus = (fw == get_focus_window()) ? FW_W(fw) : None;
 
 	/* calculate various geometries */
@@ -1848,7 +1914,7 @@ void frame_free_move_resize_args(
 			dpy, FW_W_FRAME(fw), FShapeBounding, 0, 0, None,
 			FShapeSet);
 	}
-	frame_setup_shape(fw, mra->end_g.width, mra->end_g.height);
+	frame_setup_shape(fw, mra->end_g.width, mra->end_g.height, fw->wShaped);
 	if (mra->flags.do_restore_gravity)
 	{
 		mra->grav.client_grav = fw->hints.win_gravity;
@@ -1972,7 +2038,7 @@ void frame_force_setup_window(
  * Sets up the shaped window borders
  *
  ****************************************************************************/
-void frame_setup_shape(FvwmWindow *fw, int w, int h)
+void frame_setup_shape(FvwmWindow *fw, int w, int h, int shape_mode)
 {
 	XRectangle rect;
 	rectangle r;
@@ -1982,35 +2048,42 @@ void frame_setup_shape(FvwmWindow *fw, int w, int h)
 	{
 		return;
 	}
-	if (!fw->wShaped)
+	if (fw->wShaped != shape_mode)
+	{
+		fw->wShaped = shape_mode;
+		frame_reshape_border(fw);
+	}
+	if (!shape_mode)
 	{
 		/* unset shape */
 		FShapeCombineMask(
 			dpy, FW_W_FRAME(fw), FShapeBounding, 0, 0, None,
 			FShapeSet);
-		return;
 	}
-	/* shape the window */
-	get_window_borders(fw, &b);
-	FShapeCombineShape(
-		dpy, FW_W_FRAME(fw), FShapeBounding, b.top_left.width,
-		b.top_left.height, FW_W(fw), FShapeBounding, FShapeSet);
-	if (FW_W_TITLE(fw))
+	else
 	{
-		/* windows w/ titles */
-		r.width = w;
-		r.height = h;
-		get_title_geometry(fw, &r);
-		rect.x = r.x;
-		rect.y = r.y;
-		rect.width = r.width;
-		rect.height = r.height;
-		FShapeCombineRectangles(
-			dpy, FW_W_FRAME(fw), FShapeBounding, 0, 0, &rect, 1,
-			FShapeUnion, Unsorted);
+		/* shape the window */
+		get_window_borders(fw, &b);
+		FShapeCombineShape(
+			dpy, FW_W_FRAME(fw), FShapeBounding, b.top_left.width,
+			b.top_left.height, FW_W(fw), FShapeBounding, FShapeSet);
+		if (FW_W_TITLE(fw))
+		{
+			/* windows w/ titles */
+			r.width = w;
+			r.height = h;
+			get_title_geometry(fw, &r);
+			rect.x = r.x;
+			rect.y = r.y;
+			rect.width = r.width;
+			rect.height = r.height;
+			FShapeCombineRectangles(
+				dpy, FW_W_FRAME(fw), FShapeBounding, 0, 0,
+				&rect, 1, FShapeUnion, Unsorted);
+		}
 	}
 
 	return;
 }
 
-/* ---------------------------- builtin commands ---------------------------- */
+/* ---------------------------- builtin commands --------------------------- */
