@@ -14,17 +14,17 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/****************************************************************************
+/*
  * This module is all original code
  * by Rob Nation
  * Copyright 1993, Robert Nation
  *     You may use this code for any purpose, as long as the original
  *     copyright remains in the source code and all documentation
- ****************************************************************************/
+ */
 
-/* ---------------------------- included header files ----------------------- */
+/* ---------------------------- included header files ---------------------- */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 #include <signal.h>
@@ -49,6 +49,8 @@
 #include "libs/PictureBase.h"
 #include "libs/PictureUtils.h"
 #include "libs/FRenderInit.h"
+#include "libs/charmap.h"
+#include "libs/wcontext.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "cursor.h"
@@ -77,7 +79,7 @@
 #include "menus.h"
 #include "libs/FGettext.h"
 
-/* ---------------------------- local definitions --------------------------- */
+/* ---------------------------- local definitions -------------------------- */
 
 #define MAXHOSTNAME 255
 #define MAX_CFG_CMDS 10
@@ -90,15 +92,15 @@
 #define s_g_width 4
 #define s_g_height 4
 
-/* ---------------------------- local macros -------------------------------- */
+/* ---------------------------- local macros ------------------------------- */
 
-/* ---------------------------- imports ------------------------------------- */
+/* ---------------------------- imports ------------------------------------ */
 
 extern int last_event_type;
 
-/* ---------------------------- included code files ------------------------- */
+/* ---------------------------- included code files ------------------------ */
 
-/* ---------------------------- local types --------------------------------- */
+/* ---------------------------- local types -------------------------------- */
 
 typedef enum
 {
@@ -107,9 +109,9 @@ typedef enum
 	FVWM_RESTART
 } fvwm_run_state_t;
 
-/* ---------------------------- forward declarations ------------------------ */
+/* ---------------------------- forward declarations ----------------------- */
 
-/* ---------------------------- local variables ----------------------------- */
+/* ---------------------------- local variables ---------------------------- */
 
 static char *config_commands[MAX_CFG_CMDS];
 static int num_config_commands=0;
@@ -134,7 +136,7 @@ static const char *initFunctionNames[4] =
 	"Nop"
 };
 
-/* ---------------------------- exported variables (globals) ---------------- */
+/* ---------------------------- exported variables (globals) --------------- */
 
 int master_pid;                 /* process number of 1st fvwm process */
 
@@ -194,13 +196,14 @@ Atom _XA_OL_DECOR_HEADER;
 Atom _XA_OL_DECOR_ICON_NAME;
 
 Atom _XA_WM_WINDOW_ROLE;
+Atom _XA_WINDOW_ROLE;
 Atom _XA_WM_CLIENT_LEADER;
 Atom _XA_SM_CLIENT_ID;
 
 Atom _XA_XROOTPMAP_ID;
 Atom _XA_XSETROOT_ID;
 
-/* ---------------------------- local functions ----------------------------- */
+/* ---------------------------- local functions ---------------------------- */
 
 static void SaveDesktopState()
 {
@@ -225,9 +228,7 @@ static void SaveDesktopState()
 
 static void InternUsefulAtoms (void)
 {
-	/*
-	 * Create priority colors if necessary.
-	 */
+	/* Create priority colors if necessary. */
 	_XA_MIT_PRIORITY_COLORS = XInternAtom(
 		dpy, "_MIT_PRIORITY_COLORS", False);
 	_XA_WM_CHANGE_STATE = XInternAtom(dpy, "WM_CHANGE_STATE", False);
@@ -240,7 +241,6 @@ static void InternUsefulAtoms (void)
 	_XA_WM_DESKTOP = XInternAtom(dpy, "WM_DESKTOP", False);
 	_XA_MwmAtom=XInternAtom(dpy, "_MOTIF_WM_HINTS",False);
 	_XA_MOTIF_WM=XInternAtom(dpy, "_MOTIF_WM_INFO",False);
-
 	_XA_OL_WIN_ATTR=XInternAtom(dpy, "_OL_WIN_ATTR",False);
 	_XA_OL_WT_BASE=XInternAtom(dpy, "_OL_WT_BASE",False);
 	_XA_OL_WT_CMD=XInternAtom(dpy, "_OL_WT_CMD",False);
@@ -253,11 +253,10 @@ static void InternUsefulAtoms (void)
 	_XA_OL_DECOR_RESIZE=XInternAtom(dpy, "_OL_DECOR_RESIZE",False);
 	_XA_OL_DECOR_HEADER=XInternAtom(dpy, "_OL_DECOR_HEADER",False);
 	_XA_OL_DECOR_ICON_NAME=XInternAtom(dpy, "_OL_DECOR_ICON_NAME",False);
-
 	_XA_WM_WINDOW_ROLE=XInternAtom(dpy, "WM_WINDOW_ROLE",False);
+	_XA_WINDOW_ROLE=XInternAtom(dpy, "WINDOW_ROLE",False);
 	_XA_WM_CLIENT_LEADER=XInternAtom(dpy, "WM_CLIENT_LEADER",False);
 	_XA_SM_CLIENT_ID=XInternAtom(dpy, "SM_CLIENT_ID",False);
-
 	_XA_XROOTPMAP_ID=XInternAtom(dpy, "_XROOTPMAP_ID",False);
 	_XA_XSETROOT_ID=XInternAtom(dpy, "_XSETROOT_ID",False);
 
@@ -280,9 +279,9 @@ static void catch_exit(void)
 	return;
 }
 
-/*************************************************************************
+/*
  * Restart on a signal
- ************************************************************************/
+ */
 static RETSIGTYPE
 Restart(int sig)
 {
@@ -293,7 +292,7 @@ Restart(int sig)
 	 * BEFORE we call it ... */
 	fvwmSetTerminate(sig);
 
-	return;
+	SIGNAL_RETURN;
 }
 
 static RETSIGTYPE
@@ -306,7 +305,7 @@ SigDone(int sig)
 	 * BEFORE we call it ... */
 	fvwmSetTerminate(sig);
 
-	return;
+	SIGNAL_RETURN;
 }
 
 /*
@@ -490,12 +489,45 @@ static int parseCommandArgs(
 	return errorCode ? errorCode : argc;
 }
 
-/***********************************************************************
+/*
+ *
+ */
+static
+char *get_display_name(char *display_name, int screen_num)
+{
+	char *msg;
+	char *new_dn;
+	char *cp;
+	char string_screen_num[32];
+
+	CopyString(&msg, display_name);
+	cp = strchr(msg, ':');
+	if (cp != NULL)
+	{
+		cp = strchr(cp, '.');
+		if (cp != NULL)
+		{
+			/* truncate at display part */
+			*cp = '\0';
+		}
+	}
+	sprintf(string_screen_num, ".%d", screen_num);
+	new_dn = safemalloc(
+		strlen(msg) + strlen(string_screen_num) + 1);
+	new_dn[0] = '\0';
+	strcat(new_dn, msg);
+	strcat(new_dn, string_screen_num);
+	free(msg);
+
+	return new_dn;
+}
+
+/*
  *
  *  Procedure:
  *      Done - tells FVWM to clean up and exit
  *
- ***********************************************************************/
+ */
 /* if restart is true, command must not be NULL... */
 void Done(int restart, char *command)
 {
@@ -680,11 +712,11 @@ void Done(int restart, char *command)
 	exit(0);
 }
 
-/*************************************************************************
+/*
  * Reap child processes, preventing them from becoming zombies.
  * We do this asynchronously within the SIGCHLD handler so that
  * "it just happens".
- ************************************************************************/
+ */
 static RETSIGTYPE
 ReapChildren(int sig)
 {
@@ -713,7 +745,7 @@ ReapChildren(int sig)
 #endif
 	BSD_UNBLOCK_SIGNALS;
 
-	return;
+	SIGNAL_RETURN;
 }
 
 /***********************************************************************
@@ -758,30 +790,25 @@ InstallSignals(void)
 	sigaction(SIGQUIT, &sigact, NULL);
 	sigaction(SIGTERM, &sigact, NULL);
 
-	/*
-	 * Reap all zombies automatically! This signal handler will
-	 * only be called if a child process dies, not if someone
-	 * sends a child a STOP signal. Note that none of our "terminate"
-	 * signals can be delivered until the SIGCHLD handler completes,
-	 * and this is a Good Thing because the terminate handlers
-	 * might exit abruptly via "siglongjmp". This could potentially
-	 * leave SIGCHLD handler with unfinished business ...
+	/* Reap all zombies automatically! This signal handler will only be
+	 * called if a child process dies, not if someone sends a child a STOP
+	 * signal. Note that none of our "terminate" signals can be delivered
+	 * until the SIGCHLD handler completes, and this is a Good Thing
+	 * because the terminate handlers might exit abruptly via "siglongjmp".
+	 * This could potentially leave SIGCHLD handler with unfinished
+	 * business ...
 	 *
-	 * NOTE: We could still receive SIGPIPE signals within the
-	 *       SIGCHLD handler, but the SIGPIPE handler has the
-	 *       SA_RESTART flag set and so should not affect our
-	 *       "wait" system call.
-	 */
+	 * NOTE:  We could still receive SIGPIPE signals within the SIGCHLD
+	 * handler, but the SIGPIPE handler has the SA_RESTART flag set and so
+	 * should not affect our "wait" system call. */
 	sigact.sa_flags |= SA_NOCLDSTOP;
 	sigact.sa_handler = ReapChildren;
 	sigaction(SIGCHLD, &sigact, NULL);
 #else
 #ifdef USE_BSD_SIGNALS
-	fvwmSetSignalMask( sigmask(SIGUSR1) |
-			   sigmask(SIGINT)  |
-			   sigmask(SIGHUP)  |
-			   sigmask(SIGQUIT) |
-			   sigmask(SIGTERM) );
+	fvwmSetSignalMask(
+		sigmask(SIGUSR1) | sigmask(SIGINT) | sigmask(SIGHUP) |
+		sigmask(SIGQUIT) | sigmask(SIGTERM) );
 #endif
 
 	/*
@@ -1235,7 +1262,7 @@ static void usage(int is_verbose)
 		" [-d display]"
 		" [-f cfgfile]"
 		" [-c cmd]"
-		" [-s]"
+		" [-s [screen_num]]"
 		" [-I vis-id | -C vis-class]"
 		" [-l colors"
 		" [-L|A|S|P] ...]"
@@ -1264,7 +1291,7 @@ static void usage(int is_verbose)
 		" -L:           strict color limit\n"
 		" -P:           visual palette\n"
 		" -r:           replace running window manager\n"
-		" -s:           manage a single screen\n"
+		" -s [screen]:  manage a single screen\n"
 		" -S:           static palette\n"
 		" -V:           print version information\n"
 		);
@@ -1471,7 +1498,7 @@ static int FvwmErrorHandler(Display *dpy, XErrorEvent *event)
 	return 0;
 }
 
-/* ---------------------------- interface functions ------------------------- */
+/* ---------------------------- interface functions ------------------------ */
 
 /* Does initial window captures and runs init/restart function */
 void StartupStuff(void)
@@ -1683,8 +1710,8 @@ int main(int argc, char **argv)
 	int i;
 	int len;
 	char *display_string;
-	char message[255];
 	Bool do_force_single_screen = False;
+	int single_screen_num = -1;
 	Bool replace_wm = False;
 	int visualClass = -1;
 	int visualId = -1;
@@ -1797,6 +1824,16 @@ int main(int argc, char **argv)
 			 strcmp(argv[i], "--single-screen") == 0)
 		{
 			do_force_single_screen = True;
+			if (i+1 < argc && argv[i+1][0] != '-')
+			{
+				i++;
+				if (sscanf(argv[i], "%d", &single_screen_num) ==
+				    0)
+				{
+					usage(0);
+					exit(1);
+				} 
+			}
 		}
 		else if (strcmp(argv[i], "-d") == 0 ||
 			 strcmp(argv[i], "-display") == 0 ||
@@ -1996,48 +2033,101 @@ int main(int argc, char **argv)
 	DBUG("main", "Installing signal handlers");
 	InstallSignals();
 
-	if (!(Pdpy = dpy = XOpenDisplay(display_name)))
+	if (single_screen_num >= 0)
 	{
-		fvwm_msg(
-			ERR, "main", "can't open display %s",
-			XDisplayName(display_name));
-		exit (1);
-	}
-	atexit(catch_exit);
-	Scr.screen= DefaultScreen(dpy);
-	Scr.NumberOfScreens = ScreenCount(dpy);
+		char *dn = NULL;
 
+		if (display_name)
+		{
+			dn = display_name;
+		}
+		if (!dn)
+		{
+			dn = getenv("DISPLAY");
+		}
+		if (!dn)
+		{
+			/* should never happen ? */
+			if (!(dpy = XOpenDisplay(dn)))
+			{
+				fvwm_msg(
+					ERR, "main", "can't open display %s"
+					"to get the default display",
+					XDisplayName(dn));
+			}
+			else
+			{
+				dn = XDisplayString(dpy);
+			}
+		}
+		if (dn == NULL)
+		{
+			fvwm_msg(
+				ERR, "main", "Cannot found default display (%s)",
+				XDisplayName(dn));
+		}
+		else
+		{
+			char *new_dn;
+			
+			new_dn = get_display_name(dn, single_screen_num);
+			if (dpy && strcmp(new_dn, dn) == 0)
+			{
+				/* allready opened */
+				Pdpy = dpy;
+			}
+			else if (dpy)
+			{
+				XCloseDisplay(dpy);
+				dpy = NULL;
+			}
+			if (!dpy && !(Pdpy = dpy = XOpenDisplay(new_dn)))
+			{
+				fvwm_msg(
+					ERR, "main",
+					"can't open display %s, single screen "
+					"number %d maybe not correct",
+					new_dn, single_screen_num);
+			}
+			Scr.screen = single_screen_num;
+			Scr.NumberOfScreens = ScreenCount(dpy);
+			free(new_dn);
+		}
+	}
+
+	if (!dpy)
+	{
+		if(!(Pdpy = dpy = XOpenDisplay(display_name)))
+		{
+			fvwm_msg(
+				ERR, "main", "can't open display %s",
+				XDisplayName(display_name));
+			exit (1);
+		}
+		Scr.screen= DefaultScreen(dpy);
+		Scr.NumberOfScreens = ScreenCount(dpy);
+	}
+
+	atexit(catch_exit);
 	master_pid = getpid();
 
 	if (!do_force_single_screen)
 	{
 		int myscreen = 0;
-		char *cp;
+		char *new_dn;
+		char *dn;
 
-		strcpy(message, XDisplayString(dpy));
+		dn = XDisplayString(dpy);
 		for (i=0;i<Scr.NumberOfScreens;i++)
 		{
 			if (i != Scr.screen && fork() == 0)
 			{
 				myscreen = i;
-				/* Truncate the string 'whatever:n.n' to
-				 * 'whatever:n', and then append the screen
-				 * number. */
-				cp = strchr(message, ':');
-				if (cp != NULL)
-				{
-					cp = strchr(cp, '.');
-					if (cp != NULL)
-					{
-						/* truncate at display part */
-						*cp = '\0';
-					}
-				}
-				sprintf(message + strlen(message), ".%d",
-					myscreen);
-				Pdpy = dpy = XOpenDisplay(message);
+				new_dn = get_display_name(dn, myscreen);
+				Pdpy = dpy = XOpenDisplay(new_dn);
 				Scr.screen = myscreen;
 				Scr.NumberOfScreens = ScreenCount(dpy);
+				free(new_dn);
 
 				break;
 			}
