@@ -1,3 +1,4 @@
+/* -*-c-*- */
 /*
  * FvwmButtons, copyright 1996, Jarl Totland
  *
@@ -12,21 +13,6 @@
  *
  */
 
-/* This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
-
 /* -------------------------------- DEBUG ---------------------------------- */
 
 /* Uncomment some of these defines to get (lots of) debug output. If you can't
@@ -157,8 +143,8 @@ struct button_info_struct
 {
   /* required fields */
   unsigned long flags;
-  int  BPosX,BPosY;        /* position in button units from top left */
-  byte BWidth,BHeight;     /* width and height in button units  */
+  int  BPosX,BPosY;               /* position in button units from top left */
+  unsigned int BWidth,BHeight;     /* width and height in button units  */
   button_info *parent;
   int n;                   /* number in parent */
 
@@ -239,6 +225,7 @@ char *mymalloc(int);
 void handle_xinerama_string(char *args);
 int LoadIconFile(const char *s, FvwmPicture **p, int cset);
 void SetTransparentBackground(button_info *ub,int w,int h);
+void exec_swallow(char *action, button_info *b);
 
 /* ----------------------------- global variables -------------------------- */
 
