@@ -1,20 +1,4 @@
-/* This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
-
-#include "config.h"
-
+/* -*-c-*- */
 #include <assert.h>
 #include <stdio.h>
 
@@ -23,11 +7,13 @@
 #include <X11/Xlib.h>
 #include <X11/Intrinsic.h>
 
-#include <libs/Flocale.h>
-#include <libs/Picture.h>
-#include <libs/Colorset.h>
-
 #include "libs/fvwmlib.h"
+#include "libs/Flocale.h"
+#include "libs/Picture.h"
+#include "libs/Colorset.h"
+#include "libs/Bindings.h"
+#include "libs/charmap.h"
+#include "libs/modifiers.h"
 
 #include "debug.h"
 #include "fvwm/fvwm.h"
@@ -198,6 +184,7 @@ typedef struct win_data {
 	Ulong desknum;
 	long x, y, width, height;
 	rectangle icon_g;
+	rectangle real_g; /* geometry of the client possibliy shaded */
 	Ulong app_id;
 	/* Ulong fvwm_flags; */
 	window_flags flags;
@@ -339,6 +326,7 @@ typedef struct win_manager {
 	/* fvwm state */
 	int we_are_drawing, configures_expected;
 	Bool swallowed;
+	Window swallower_win;
 	struct
 	{
 		unsigned is_shaded : 1;
@@ -384,7 +372,7 @@ extern void ReadFvwmPipe(void);
 extern void *Malloc (size_t size);
 extern void Free (void *p);
 extern void ShutMeDown (int flag) __attribute__ ((__noreturn__));
-extern void DeadPipe (int nothing) __attribute__ ((__noreturn__));
+extern RETSIGTYPE DeadPipe (int nothing);
 extern char *copy_string (char **target, const char *src);
 
 extern void init_globals (void);
@@ -409,3 +397,5 @@ extern void print_managers (void);
 
 extern WinManager *find_windows_manager (Window win);
 extern int win_in_viewport (WinData *win);
+extern WinData *id_to_win(Ulong id);
+
